/************
 *
 *	connectsock.c
 *
 ************
*
*
*  allocate a socket and connect it.
*  the caller specifies whether to use UDP or TCP.
*
* $Id: connectsock.cpp,v 1.2 2006/01/02 06:07:08 greg Exp $
*
*/

#include "stdafx.h"
#include "sockipc.h"

#ifndef INADDR_NONE
#define INADDR_NONE    0xffffffff
#endif  /* INADDR_NONE */

/*
 **
 **		c o n n e c t U D P
 **
 **	connect to a specified UDP service on a specified host
 **
*/
int
connectUDP(char *host, char *service)
{
  return(connectsock(host, service, "udp"));
}

/*
 **
 **		c o n n e c t T C P
 **
 **	connect to a specified TCP service on a specified host
 **
*/
int
connectTCP(char *host, char *service)
{
  return(connectsock(host, service, "tcp"));
}

/*
 **
 **		c o n n e c t s o c k
 **
 **	allocate and connect a socket using TCP or UDP
 **
*/
int
connectsock(char *host, char *service, char *protocol)
{
	struct hostent *phe;     /* pointer to host information entry */
	struct servent *pse;     /* pointer to service information entry */
	struct protoent *ppe;    /* pointer to protocol information entry */
	struct sockaddr_in sin;  /* an Internet endpoint address */
	int s, type;             /* socket descriptor and socket type */

	memset((char *)&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;

	/* map service name to port number */
	if(pse = getservbyname(service, protocol))
		sin.sin_port = pse->s_port;
	else if((sin.sin_port = htons((u_short)atoi(service))) == 0)
		fprintf(stderr, "can't get \"%s\" service entry\n", service);
	
	/* map host name to IP address, allowing for dotted decimal */
	if(phe = gethostbyname(host))
		memcpy((char *)&sin.sin_addr, phe->h_addr, phe->h_length);
	else if((sin.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
	{
		fprintf(stderr, "can't get \"%s\" host entry\n", host);
		return -1;
	}

	/* map protocol name to protocol number */
	if((ppe = getprotobyname(protocol)) == 0)
	{
		fprintf(stderr, "can't get \"%s\" protocol entry\n", protocol);
		return -2;
	}

	/* use protocol to choose a socket type */
	if(strcmp(protocol, "udp") == 0)
		type = SOCK_DGRAM;
	else
		type = SOCK_STREAM;

	/* allocate a socket */
	if((s = socket(PF_INET, type, ppe->p_proto)) <= 0)
	{
		fprintf(stderr, "can't create socket: %s:%d\n", SLE);
		return -3;
	}

	/* connect the socket */
	errno = 0;
	if(connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	{
		fprintf(stderr, "can't connect to %s.%s: %s:%d\n", host, service, SLE);
		s = -4;
	}
	return(s);
}

